<?php
/*
 * #? This file contains the admin panel options
 */

/**
 * Renders menu and proccess settings
 *
 * @package Wordpress
 * @since 1.0
 */
class SetingsRenderer {

    function __construct() {
        add_action('admin_menu', array(
            $this,
            'SettingsMenu'
        ));
        add_action('admin_init', array(
            $this,
            'ErpAdminInit'
        ));
        add_action('admin_enqueue_scripts', array(
            $this,
            'erp_jq_scripts'
        ));
    }

    /**
     * Registers options menu
     *
     * @package Wordpress
     * @since 1.0
     */
    function SettingsMenu() {
        add_options_page('Easy related posts configuration', 'Easy Related Posts', 'manage_options', 'easy_related_posts', array(
            $this,
            'OptPageRenderer'
        ));
    }

    /**
     * Registers options update function
     *
     * @package Wordpress
     * @since 1.0
     */
    function ErpAdminInit() {
        add_action('admin_post_save_erpOpts', array(
            $this,
            'OptsProcessing'
        ));
    }

    /**
     * Loads jquerry scripts
     *
     * @package Wordpress
     * @since 1.0
     */
    function erp_jq_scripts() {
        wp_enqueue_script('jquery');
        wp_enqueue_script('jquery-ui-core');
        wp_enqueue_style('wp-color-picker');
        wp_enqueue_script('wp-color-picker');
        wp_enqueue_script('jquery-effects-fade');
//        wp_enqueue_script('jquery-ui-accordion');
        wp_enqueue_script('jquery-ui-tabs');
        wp_enqueue_script('erpAdminScripts', plugins_url('/inc/js/adminScripts.js', __FILE__));
        wp_enqueue_style('qtipStyle');
        wp_enqueue_script('qtip', plugins_url('/inc/js/jquery.qtip.min.js', __FILE__), array(
            'jquery'
                ), false, true);
    }

    /**
     * Renders given options as table elements.
     * Html table opening and closing tags must be initialized outside function.
     *
     * @package WordPress
     * @since 1.0
     *       
     * @param string $optType
     *        	Text, checkbox, etc
     * @param string $optName
     *        	Unique name
     * @param string $optID
     *        	Unique ID
     * @param string $optValue
     *        	Current value
     * @param string $optMultiple
     *        	Array with choises in case of multiple choises ('dropdown' or 'radio') or with html element atributes in other cases
     * @return string
     */
    function OptArrayRenderer($optType, $optName, $optID, $optMultiple = FALSE) {
        $opt = get_option('erpOpts');

        if ($optType == 'checkbox') {
            if (is_array($optMultiple)) {
                $atrs = '';
                foreach ($optMultiple as $atr => $value) {
                    $atrs .= ' ' . $atr . '="' . $value . '" ';
                }
            } else {
                $atrs = '';
            }
            $output = '<tr><td><label for="' . $optID . '">' . $optName . ' :' . '</label></td>';
            $output .= '<td><input class="erp-optchbx" id="' . $optID . '" name="' . $optID . '" type="' . $optType . '"' . 'value="' . $opt [$optID] . '" ' . $atrs . checked($opt [$optID], '1', FALSE) . '/></td></tr>';
        } elseif ($optType == 'text' || $optType == 'number') {
            if (is_array($optMultiple)) {
                $atrs = '';
                foreach ($optMultiple as $atr => $value) {
                    $atrs .= ' ' . $atr . '="' . $value . '" ';
                }
            } else {
                $atrs = '';
            }
            $output = '<tr><td><label for="' . $optID . '">' . $optName . ' :' . '</label></td>';
            $output .= '<td><input class="erp-opttxt" id="' . $optID . '" name="' . $optID . '" type="' . $optType . '"' . $atrs . 'value="' . $opt [$optID] . '"/></td></tr>';
        } elseif ($optType == 'select' && is_array($optMultiple)) {
            $id = $opt [$optID];
            $output = '<tr><td><label for="' . $optID . '">' . $optName . ' :' . '</td>';
            $output .= '<td><select class="erp-optsel" id="' . $optID . '" name="' . $optID . '">';
            foreach ($optMultiple as $key => $val) {
                $valLow = strtolower(str_replace(' ', '_', $val));
                $output .= '<option value="' . $valLow . '"' . selected($id, $valLow, FALSE) . '>' . $val . '</option>';
            }
            $output .= '</select></td></tr></label>';
        } elseif ($optType == 'radio' && is_array($optMultiple)) {
            $output = '<tr><td><label for="' . $optID . '">' . $optName . ' :' . '</label></td><td></td></tr>';
            foreach ($optMultiple as $val) {
                $valLow = strtolower(str_replace(' ', '_', $val));
                $output .= '<tr><td></td><td><input type="' . $optType . '" name="' . $val . '" value="' . checked($opt [$optID], $val, FALSE) . '"/></td></tr>';
            }
        }
        return $output;
    }

    /**
     * Renders the settings page
     *
     * @package Wordpress
     * @since 1.0
     */
    function OptPageRenderer() {
        $opt = get_option('erpOpts');

        $opt ['titletd'] = isset($opt ['titletd']) ? $opt ['titletd'] : 'Easy related posts';
        $opt ['activate_plugin'] = isset($opt ['activate_plugin']) ? $opt ['activate_plugin'] : 1;
        $opt ['num_of_p_t_dspl'] = isset($opt ['num_of_p_t_dspl']) ? $opt ['num_of_p_t_dspl'] : 5;
        $opt ['getPostsBy'] = isset($opt ['getPostsBy']) ? $opt ['getPostsBy'] : 'categories';
        /* #? FIXME Right now it seems there is a problem when getting the excerpt with get_the_excerpt() or any related func */
        $opt ['erpcontent'] = isset($opt ['erpcontent']) ? $opt ['erpcontent'] : 'post_title';
        $opt ['exc_len'] = isset($opt ['exc_len']) ? $opt ['exc_len'] : 15;
        $opt ['ttl_sz'] = isset($opt ['ttl_sz']) ? $opt ['ttl_sz'] : 0;
        $opt ['exc_sz'] = isset($opt ['exc_sz']) ? $opt ['exc_sz'] : 0;
        $opt ['more_txt'] = isset($opt ['more_txt']) ? $opt ['more_txt'] : ' ...read more';
        $opt ['num_of_p_p_row'] = isset($opt ['num_of_p_p_row']) ? $opt ['num_of_p_p_row'] : 3;
        $opt ['popPos'] = isset($opt ['popPos']) ? $opt ['popPos'] : 'bottom';
        $opt ['popColor'] = isset($opt ['popColor']) ? $opt ['popColor'] : '#383838';
        $opt ['popTriger'] = (isset($opt ['popTriger']) && $opt ['popTriger'] <= 1 && $opt ['popTriger'] >= 0) ? $opt ['popTriger'] : '0.85';
        $opt ['popBkg'] = (isset($opt ['popBkg']) && $opt ['popBkg'] <= 1 && $opt ['popBkg'] >= 0) ? $opt ['popBkg'] : '0.65';
        $opt ['display_layout'] = isset($opt ['display_layout']) ? $opt ['display_layout'] : '1_post_per_row';
        $opt ['display_thumbnail'] = isset($opt ['display_thumbnail']) ? $opt ['display_thumbnail'] : 1;
        $opt ['crop_thumbnail'] = isset($opt ['crop_thumbnail']) ? $opt ['crop_thumbnail'] : 1;
        $opt ['thumbnail_height'] = isset($opt ['thumbnail_height']) ? $opt ['thumbnail_height'] : 150;
        $opt ['thumbnail_width'] = isset($opt ['thumbnail_width']) ? $opt ['thumbnail_width'] : '500';
        $opt ['categories'] = isset($opt ['categories']) ? $opt ['categories'] : array();
        $opt ['tags'] = isset($opt ['tags']) ? $opt ['tags'] : array();

        update_option('erpOpts', $opt);
        ?>
        <div id="erp-opt-general" class="wrap">
            <h2>Easy Related Posts Settings</h2>
            <form method="post" action="admin-post.php">

                <input type="hidden" name="action" value="save_erpOpts" />
                <?php echo wp_nonce_field('erpoptnonce'); ?>
                <div id="tabs-holder">
                    <ul>
                        <li><a href="#tabs-1">General Options</a></li>
                        <li><a href="#tabs-2">Content Options</a></li>
                        <li><a href="#tabs-3">Layout Options</a></li>
                        <li><a href="#tabs-4">Categories</a></li>
                        <li><a href="#tabs-5">Tags</a></li>
                    </ul>
                    <!--<div id="tableWraper">-->
                    <div id="tabs-1">
                        <!--				<h3>General Options</h3>-->
                        <table class="gen-opt-table">
                            <tr>
                                <th colspan="2">General Options</th>
                            </tr>
                            <?php
                            echo $this->OptArrayRenderer('checkbox', 'Activate plugin', 'activate_plugin');
                            echo $this->OptArrayRenderer('text', 'Title to display', 'titletd');
                            echo $this->OptArrayRenderer('select', 'Get posts by', 'getPostsBy', array(
                                'Categories',
                                'Tags'
                            ));
                            ?>
                        </table>
                    </div>
                    <div id="tabs-2">
                        <!--<h3>Content Options</h3>-->
                        <table class="con-opt-table">
                            <tr>
                                <th colspan="2">Content Options</th>
                            </tr>
                            <?php
                            echo $this->OptArrayRenderer('number', 'Number of posts to display', 'num_of_p_t_dspl', array(
                                'size' => '2pt'
                            ));
                            /* #? FIXME Right now it seems there is a problem when getting the excerpt with get_the_excerpt() or any related func */
                            echo $this->OptArrayRenderer('select', 'Content', 'erpcontent', array(
                                'Post title',
                                'Title + excerpt'
                            ));
                            echo $this->OptArrayRenderer('number', 'Post title size (px)', 'ttl_sz', array(
                                'size' => '2pt'
                            ));
                            echo $this->OptArrayRenderer('number', 'Post excerpt size (px)', 'exc_sz', array(
                                'size' => '2pt'
                            ));
                            echo $this->OptArrayRenderer('number', 'Excerpt length (words)', 'exc_len', array(
                                'size' => '2pt'
                            ));
                            echo $this->OptArrayRenderer('text', 'Read more text', 'more_txt');
                            ?>
                        </table>
                    </div>
                    <div id="tabs-3">
                        <!--<h3>Layout Options</h3>-->
                        <table class="lay-opt-table">
                            <tr>
                                <th colspan="2">Layout Options</th>
                            </tr>
                            <?php
                            echo $this->OptArrayRenderer('checkbox', 'Display thumbnail', 'display_thumbnail');
                            echo $this->OptArrayRenderer('checkbox', 'Crop thumbnail', 'crop_thumbnail');
                            echo $this->OptArrayRenderer('number', 'Thumbnail height', 'thumbnail_height', array(
                                'size' => '2pt'
                            ));
                            echo $this->OptArrayRenderer('number', 'Thumbnail width (optional)', 'thumbnail_width', array(
                                'size' => '2pt'
                            ));
                            echo $this->OptArrayRenderer('select', 'Display layout', 'display_layout', array(
                                '1 post per row',
                                'define bellow',
                                'pop up block'
                            ));
                            echo $this->OptArrayRenderer('number', 'Number of posts per row', 'num_of_p_p_row', array(
                                'size' => '2pt'
                            ));
                            echo $this->OptArrayRenderer('select', 'Pop up position', 'popPos', array(
                                'top',
                                'bottom'
                            )); // TODO
                            echo $this->OptArrayRenderer('text', 'Pop up block background color (hex)', 'popColor', array(
                                'size' => '6pt'
                            )); // TODO
                            echo $this->OptArrayRenderer('number', 'Background transparency (0 to 1 float)', 'popBkg', array(
                                'size' => '2pt',
                                'step' => "0.1",
                                'min' => '0',
                                'max' => '1'
                            )); // TODO
                            echo $this->OptArrayRenderer('number', 'Appear bellow (0 to 1 float)', 'popTriger', array(
                                'size' => '2pt',
                                'step' => "0.1",
                                'min' => '0',
                                'max' => '1'
                            )); // TODO
                            ?>
                        </table>
                    </div>
                    <div id="tabs-4">
                        <!--<h3>Categories</h3>-->
                        <p>Any category you might choose here will be excluded from related posts. Also when user reads posts from these categories won't see any related post.<br>
                            Please note that you must select <em>Categories</em> for <em>Get posts by</em> option in <em>General options</em> tab.</p>
                        <table class="cat-opt-table">
                            <?php
                            $opts = array(
                                'hide_empty' => 0,
                            );
                            $cats = get_categories($opts);
                            $tags = get_tags($opts);
                            
                            foreach ($cats as $key => $value) {
                                ?>
                                <tr>
                                    <td><label for="categories"><?php echo $value->name; ?> :</label>
                                    </td>
                                    <td><input class="erp-optchbx" id="categories" name="categories[]"
                                               type="checkbox" value="<?php echo $value->term_id; ?>"
                                               <?php if (in_array($value->term_id, $opt['categories'])) echo 'checked="checked"'; ?> />
                                    </td>
                                </tr>
                                <?php
                            }
                            ?>
                        </table>
                    </div>
                    <div id="tabs-5">
                        <!--<h3>Tags</h3>-->
                        <p>Any tag you might choose here will be excluded from related posts. Also when user reads posts from these tags won't see any related post.<br>
                            Please note that you must select <em>Tags</em> for <em>Get posts by</em> option in <em>General options</em> tab.</p>
                        <table class="tag-opt-table">
                            <?php
                            foreach ($tags as $key => $value) {
                                ?>
                                <tr>
                                    <td><label for="tags"><?php echo $value->name; ?> :</label></td>
                                    <td><input class="erp-optchbx" id="tags" name="tags[]"
                                               type="checkbox" value="<?php echo $value->term_id; ?>"
                                               <?php if (in_array($value->term_id, $opt['tags'])) echo 'checked="checked"'; ?> />
                                    </td>
                                </tr>
                                <?php
                            }
                            ?>
                        </table>
                    </div>
                    <!--</div>-->
                </div>
                <?php
                echo get_submit_button('Update options', 'primary large', 'Save');
                
                ?>
                <input id="tab-spec" type="hidden" name="tab-spec">
                <script type="text/javascript">
                    checkThumbOptions = new ChkObj('#display_thumbnail', '#crop_thumbnail');
                    checkCrop = new ChkObj('#crop_thumbnail', '#thumbnail_height, #thumbnail_width');
                    checkContent = new ChkObj('#erpcontent', '#exc_len, #exc_sz');
                    checkPostsPerRow = new ChkObj('#display_layout', '#num_of_p_p_row');
                    checkPostsPerRow2 = new ChkObj('#display_layout', '#popPos, #popColor, #popBkg, #popTriger');

                    checkThumbOptions.checkBoxChecker();
                    checkCrop.checkBoxChecker();
                    checkContent.selectionChecker(['post_title']);
                    checkPostsPerRow.selectionChecker(['1_post_per_row']);
                    checkPostsPerRow2.selectionChecker(['1_post_per_row', 'define_bellow']);

                    jQuery(document).ready(function($) {
                        tiper('#activate_plugin', 'If you want to use only the widget</br>of this plugin, uncheck this.');
                        tiper('#titletd', 'The text that will appear above the posts');
                        tiper('#getPostsBy', 'Fetch posts based on...');
                        tiper('#num_of_p_t_dspl', 'How many posts you\'d like to display.');
                        tiper('#erpcontent', 'The text that will be displayed for each post.</br>This will go bellow thumbnail if any.');
                        tiper('#ttl_sz', 'Here you can specify the text size of post title.</br>If left zero you themes default h3 will be used.');
                        tiper('#exc_sz', 'Here you can specify the text size of post excerpt.</br>If left zero you themes default paragraph will be used.');
                        tiper('#exc_len', 'How many words post excerpt will span before the read more text.</br>Extremely usefull if you want to cut off large excerpts.');
                        tiper('#more_txt', 'The text that will apear after each post content. Default <em> ...read more</em>');
                        tiper('#display_thumbnail', 'Thumbnail are allways placed before post title.</br>Uncheck if you don\'t want them to appear at all.');
                        tiper('#crop_thumbnail', 'Use this if you want the thumbnail to be croped. \n\
                                Setting the height to some value above zero and the width to zero will result in hard croped thumbnail.</br>\n\
                                Setting both values above zero will result in soft croped, more artistic, thumbnail.');
                        tiper('#display_layout', 'Choose the display layout:</br>\n\
                                <strong>1 post per row:</strong> Just one post per row, something like a list, only seperated by a horizontal rule.</br>\n\
                                <strong>define bellow:</strong> Selecting this will allow you to display as many posts per row you want. Result will be more like a \n\
                                    table with no borders.</br>\n\
                                <strong>pop up block:</strong> A block with the posts will slide up or down, when client scrolls down after a point you\'ll have to specify.');
                        tiper('#num_of_p_p_row', 'Specify how many posts per row will be displayed. Width of each post area is depending on your main content width.');
                        tiper('#popPos', 'Choose if you want the posts to slide down from the top, or slide up from the bottom.');
                        tiper('#popColor', 'Set the background color of the pop up block. This should be a hex value. eg #b9a9c6');
                        tiper('#popBkg', 'Set the background transparency of the pop up block. 0 for fully transparent, 1 for solid appearance.');
                        tiper('#popTriger', 'Here you can set the point that pop up block will appear. This is based on post main content height.\n\
                                    A value of 0.5 means that if the center of the browser moves bellow the center of the main content area, the pop up will appear.\n\
                                    0 to appear from the top of the window, 1 to have to scroll all the way down. If the bottom of content is reached, pop up will appear \n\
                                    anyway.');


                        jQuery('#display_thumbnail').click(function() {
                            checkThumbOptions.checkBoxChecker();
                            checkCrop.checkBoxChecker();
                        });
                        jQuery('#crop_thumbnail').click(function() {
                            checkCrop.checkBoxChecker();
                        });
                        jQuery('#erpcontent').change(function() {
                            checkContent.selectionChecker(['post_title']);
                        });
                        jQuery('#display_layout').change(function() {
                            checkPostsPerRow.selectionChecker(['1_post_per_row']);
                            checkPostsPerRow2.selectionChecker(['1_post_per_row', 'define_bellow']);
                        });
                        var options = {beforeActivate: function( event, ui ) {
                                $('#tab-spec').val(ui.newPanel.attr('id').substring(5)-1);
                            }};
                        <?php
                        if (isset($_GET['tab-spec'])){
                            echo 'options.active= ' . $_GET['tab-spec'] . ';';
                        }
                        ?>
                                console.log(options.active);
                        jQuery('#tabs-holder').tabs(options);

                        $('#getPostsBy').change(function() {
                            if ($(this).val() == 'categories') {
                                $('#tabs-holder').tabs('disable', 4);
                                $('#tabs-holder').tabs('enable', 3);
                            } else {
                                $('#tabs-holder').tabs('disable', 3);
                                $('#tabs-holder').tabs('enable', 4);
                            }
                        });
                        $('#getPostsBy').trigger('change');


                    });
                </script>
            </form>
        </div>
        <?php
    }

    /**
     * Process updated settings to be saved in options array
     *
     * @package Wordpress
     * @since 1.0
     */
    function OptsProcessing() {
        if (!current_user_can('manage_options')) {
            wp_die('Not allowed');
        }

        // #? Check this comes from admin panel
        check_admin_referer('erpoptnonce');

        $oldOpt = get_option('erpOpts');
        /* #? Fill options not included in admin panel */
        $newOpt = $oldOpt;

        $newOpt ['titletd'] = isset($_POST ['titletd']) ? strip_tags($_POST ['titletd']) : $oldOpt ['titletd'];
        $newOpt ['activate_plugin'] = isset($_POST ['activate_plugin']) ? (bool) 1 : 0;
        $newOpt ['useCategories'] = isset($_POST ['useCategories']) ? (bool) 1 : 0;
        $newOpt ['useTags'] = isset($_POST ['useTags']) ? (bool) 1 : 0;
        $newOpt ['getPostsBy'] = isset($_POST ['getPostsBy']) ? strip_tags($_POST ['getPostsBy']) : 'categories';
        $newOpt ['num_of_p_t_dspl'] = isset($_POST ['num_of_p_t_dspl']) ? (int) $_POST ['num_of_p_t_dspl'] : $oldOpt ['num_of_p_t_dspl'];
        $newOpt ['erpcontent'] = isset($_POST ['erpcontent']) ? strip_tags($_POST ['erpcontent']) : $oldOpt ['erpcontent'];
        $newOpt ['exc_len'] = isset($_POST ['exc_len']) ? (int) $_POST ['exc_len'] : $oldOpt ['exc_len'];
        $newOpt ['exc_sz'] = isset($_POST ['exc_sz']) ? (int) $_POST ['exc_sz'] : $oldOpt ['exc_sz'];
        $newOpt ['ttl_sz'] = isset($_POST ['ttl_sz']) ? (int) $_POST ['ttl_sz'] : $oldOpt ['ttl_sz'];
        $newOpt ['more_txt'] = isset($_POST ['more_txt']) ? strip_tags($_POST ['more_txt']) : $oldOpt ['more_txt'];
        $newOpt ['num_of_p_p_row'] = isset($_POST ['num_of_p_p_row']) ? (int) $_POST ['num_of_p_p_row'] : $oldOpt ['num_of_p_p_row'];
        $newOpt ['popPos'] = isset($_POST ['popPos']) ? strip_tags($_POST ['popPos']) : $oldOpt ['popPos'];
        $newOpt ['popColor'] = isset($_POST ['popColor']) ? strip_tags($_POST ['popColor']) : $oldOpt ['popColor'];
        $newOpt ['popTriger'] = (isset($_POST ['popTriger']) && $_POST ['popTriger'] <= 1 && $_POST ['popTriger'] >= 0) ? (float) $_POST ['popTriger'] : $oldOpt ['popTriger'];
        $newOpt ['popBkg'] = (isset($_POST ['popBkg']) && $_POST ['popBkg'] <= 1 && $_POST ['popBkg'] >= 0) ? (float) $_POST ['popBkg'] : $oldOpt ['popBkg'];
        $newOpt ['display_layout'] = isset($_POST ['display_layout']) ? strip_tags($_POST ['display_layout']) : $oldOpt ['display_layout'];
        $newOpt ['display_thumbnail'] = isset($_POST ['display_thumbnail']) ? (bool) 1 : 0;
        $newOpt ['crop_thumbnail'] = isset($_POST ['crop_thumbnail']) ? (bool) 1 : 0;
        $newOpt ['thumbnail_height'] = isset($_POST ['thumbnail_height']) ? (int) $_POST ['thumbnail_height'] : $oldOpt ['thumbnail_height'];
        $newOpt ['thumbnail_width'] = isset($_POST ['thumbnail_width']) ? (int) $_POST ['thumbnail_width'] : $oldOpt ['thumbnail_width'];
        $newOpt ['categories'] = isset($_POST ['categories']) ? $_POST ['categories'] : array();
        $newOpt ['tags'] = isset($_POST ['tags']) ? $_POST ['tags'] : array();

        update_option('erpOpts', $newOpt);
        wp_redirect(add_query_arg(array('page' => 'easy_related_posts', 'tab-spec' => wp_strip_all_tags($_POST['tab-spec'])), admin_url('options-general.php')));

        exit();
    }

}

// setingsRenderer
?>