<?php

/**
 * Adds Post_Events_Widget widget.
 */
// register ERP_Widget widget
function ErpWiInit() {
    register_widget("ERP_Widget");
}

add_action('widgets_init', 'ErpWiInit');

class ERP_Widget extends WP_Widget {

    /**
     * Register widget with WordPress.
     */
    public function __construct() {
        parent::__construct('ERP_Widget', 'Easy Related Posts', array('description' => __('Show related posts where in single post page')), array('width' => 350));
        add_action('admin_enqueue_scripts', array($this, 'erp_jq_scripts'));
    }

    /**
     * Loads jquerry scripts
     *
     * @package Wordpress
     * @since 1.0
     */
    function erp_jq_scripts() {
        wp_enqueue_script('jquery');
        wp_enqueue_script('jquery-ui-core');
        wp_enqueue_style('wp-color-picker');
        wp_enqueue_script('wp-color-picker');
        wp_enqueue_script('jquery-effects-fade');
        wp_enqueue_script('erpAdminScripts', plugins_url('/inc/js/adminScripts.js', __FILE__));
        // FIXME Maybe this should be removed, debugit
        //wp_enqueue_script( 'wp-color-picker-script', plugins_url('script.js', __FILE__ ), array( 'wp-color-picker' ), false, true );
    }

    /**
     * Front-end display of widget.
     *
     * @see WP_Widget::widget()
     *
     * @param array $args     Widget arguments.
     * @param array $instance Saved values from database.
     */
    public function widget($args, $instance) {
        // outputs the content of the widget
        extract($args);

        global $post; // Unseted bellow

        if (!is_single()) {
            return;
        } else {

            // Retrieve widget configuration options
            $wid_num_of_p_t_dspl = (!empty($instance['wid_num_of_p_t_dspl']) ? $instance['wid_num_of_p_t_dspl'] : 10 );

            $widget_title = (!empty($instance['wid_erp_title']) ? esc_attr($instance['wid_erp_title']) : 'Related posts' );
            $widTitle = apply_filters('widget_title', $widget_title);

            $getPostsBy = (!empty($instance['wid_getPostsBy']) ) ? esc_attr($instance['wid_getPostsBy']) : 'categories';
            $wid_taxExlude = (!empty($instance['wid_taxExlude'])) ? ($instance['wid_taxExlude']) : 0;

            $wid_erp_gen_o = (!empty($instance['wid_erp_gen'])) ? ($instance['wid_erp_gen']) : 0;

            $wid_erp_exc_or_tit = (!empty($instance['wid_erp_exc_or_tit'])) ? ($instance['wid_erp_exc_or_tit']) : 'post_title';

            $wid_erp_pt_s = (!empty($instance['wid_erp_pt_s']) ? $instance['wid_erp_pt_s'] : '' );
            $wid_erp_pt_c = (!empty($instance['wid_erp_pt_c']) ? $instance['wid_erp_pt_c'] : '' );
            $wid_erp_pt_c_u = (!empty($instance['wid_erp_pt_c_u']) ? $instance['wid_erp_pt_c_u'] : '' );

            $wid_erp_exc_s = (!empty($instance['wid_erp_exc_s']) ? $instance['wid_erp_exc_s'] : '' );
            $wid_erp_exc_c = (!empty($instance['wid_erp_exc_c']) ? $instance['wid_erp_exc_c'] : '' );
            $wid_erp_exc_c_u = (!empty($instance['wid_erp_exc_c_u']) ? $instance['wid_erp_exc_c_u'] : '' );

            $wid_erp_pt_color = $wid_erp_pt_c_u ? $wid_erp_pt_c : '';
            $wid_erp_exc_color = $wid_erp_exc_c_u ? $wid_erp_exc_c : '';

            $wid_erp_thumb_dis = (!empty($instance['wid_erp_thumb'])) ? ($instance['wid_erp_thumb']) : 0;
            $wid_erp_thumb_crop = (!empty($instance['wid_erp_thumb_crop']) ? $instance['wid_erp_thumb_crop'] : '' );
            $wid_erp_thumb_h = (!empty($instance['wid_erp_thumb_h']) ? $instance['wid_erp_thumb_h'] : '' );
            $wid_erp_thumb_w = (!empty($instance['wid_erp_thumb_w']) ? $instance['wid_erp_thumb_w'] : '' );
            
            $pluginOpts = get_option ( 'erpOpts' ); 
            
            if ($wid_taxExlude && $getPostsBy == 'categories' && !empty($pluginOpts['categories'])) {
                $post_categories = get_the_category();
                if (empty($post_categories)) {
                    if (!$wid_erp_gen_o){
                        echo $before_widget;
                        echo $before_title . $widTitle . $after_title;
                        echo 'No related posts found';
                        echo $after_widget;
                    }
                    return;
                }
                foreach ($post_categories as $cat) {
                    if (in_array($cat->term_id, $pluginOpts['categories'])) {
                        return;
                    }
                }
            } elseif ($wid_taxExlude && $getPostsBy == 'tags' && !empty($pluginOpts['tags'])) {
                // TODO Return if post have specified tags
                $post_tags = get_the_tags($post->ID);
                if (empty($post_tags)) {
                    if (!$wid_erp_gen_o){
                        echo $before_widget;
                        echo $before_title . $widTitle . $after_title;
                        echo 'No related posts found';
                        echo $after_widget;
                    }
                    return ;
                }
                foreach ($post_tags as $tag) {
                    if (in_array($tag->term_id, $pluginOpts['tags'])) {
                        return ;
                    }
                }
            }

            $argu = array(
                'post_status' => 'publish',
                'post_visibility' => 'public',
                'posts_per_page' => $wid_num_of_p_t_dspl,
                'ignore_sticky_posts' => 1,
                'post__not_in' => (array) $post->ID,
                'orderby' => 'date',
                'order' => 'DESC'
            );
            
            if ($getPostsBy == 'categories') {
                $catAr = array();
                foreach ((get_the_category($post->ID)) as $category) {
                    array_push($catAr, $category->cat_ID);
                }
                if (empty($catAr)){
                    if (!$wid_erp_gen_o){
                        echo $before_widget;
                        echo $before_title . $widTitle . $after_title;
                        echo 'No related posts found';
                        echo $after_widget;
                    }
                    return;
                }
                $argu['category__in'] = $catAr;
                if (!empty($pluginOpts['categories'])){
                    $argu['category__not_in'] = $pluginOpts['categories'];
                }
            } else {
                $tagAr = array();
                foreach ((get_the_tags($post->ID)) as $tags) {
                    array_push($tagAr, $tags->term_id);
                }
                if (empty($tagAr)){
                    if (!$wid_erp_gen_o){
                        echo $before_widget;
                        echo $before_title . $widTitle . $after_title;
                        echo 'No related posts found';
                        echo $after_widget;
                    }
                    return;
                }
                $argu['tag__in'] = $tagAr;
                if (!empty($pluginOpts['tags'])){
                    $argu['tag__not_in'] = $pluginOpts['tags'];
                }
            }

            unset($post);

            $the_query = new WP_Query($argu);

            
            if (!$wid_erp_gen_o || $the_query->have_posts()) {

                echo $before_widget;

                echo $before_title . $widTitle . $after_title;



                // Display posts in unordered list layout
                echo '<div id="erp_container">';
                echo '<ul class="erp_ul_elem">';
                // Cycle through all items retrieved
                while ($the_query->have_posts()) {
                    $the_query->the_post();

                    $contenttitle = '<h3 class="erp_content"  style="margin-top:0px; margin-bottom:0px; text-weight:bold; font-size:' . ($wid_erp_pt_s ? $wid_erp_pt_s . "px" : "100%") . '; color:' . $wid_erp_pt_color . '">' . get_the_title(get_the_ID()) . '</h3>';

                    $contentexc = $wid_erp_exc_or_tit == 'post_excerpt' ? '<p class="erp_content" style="margin-top:0px; margin-bottom:0px; font-size:' . ($wid_erp_exc_s ? $wid_erp_exc_s . "px" : "100%") . '; color:' . $wid_erp_exc_color . '">' . wp_trim_words(get_the_content(), 10, '<span style="font-size:80%;"><em> ...read more</em></span>') . '</p>' : '';

                    /* #? Get the thumb, medium size should be ok for most of themes */
                    if ($wid_erp_thumb_dis && $wid_erp_thumb_crop && $wid_erp_thumb_h && has_post_thumbnail(get_the_ID())) {
                        /* Fixed croped images not loading */
                        $th = wp_get_attachment_image_src(get_post_thumbnail_id(get_the_ID()), "medium");
                        $src = substr($th[0], strlen(get_bloginfo('wpurl')));
                        $th = wp_get_attachment_image_src(get_post_thumbnail_id(get_the_ID()), "medium");
                        $wid_erp_thumb_display = '<br><img style="width:100%;" src="' . plugins_url(basename(dirname(__FILE__))) . '/inc/tim/tb.php?src=' . $src . '&h=' . ($wid_erp_thumb_h > $th[2] ? $th[2] : $wid_erp_thumb_h) . '&w=' . ($wid_erp_thumb_w ? $wid_erp_thumb_w : $th[1] ) . '&q=90" />';
                    } else if (has_post_thumbnail(get_the_ID()) && $wid_erp_thumb_dis) {
                        $wid_erp_thumb_display = get_the_post_thumbnail(get_the_ID(), 'medium', array("style" => "width:100%;"));
                    } else {
                        $wid_erp_thumb_display = '';
                    }

                    echo '<li class="erp_li_elem" ><a class="erp_a_elem"  href="' . get_permalink() . '">';
                    echo $wid_erp_thumb_display . $contenttitle . $contentexc . '</a></li>';
                }
                echo '</ul>';
                echo '</div>';
            }
            wp_reset_postdata();

            echo $after_widget;
        }
    }

    /**
     * Back-end widget form.
     *
     * @see WP_Widget::form()
     *
     * @param array $instance Previously saved values from database.
     */
    public function form($instance) {
        // outputs the options form on admin
        // processes widget options to be saved
        /* @? Title of the widget */
        $wid_erp_title = (isset($instance['wid_erp_title'])) ? ( $instance['wid_erp_title']) : ( __('Easy Related Posts'));
        /* #? Fetch by category or by tags */
        $wid_getPostsBy = (isset($instance['wid_getPostsBy'])) ? ( $instance['wid_getPostsBy']) : 'categories';
        /* #? Fetch by category or by tags */
        $wid_taxExlude = (isset($instance['wid_taxExlude'])) ? ( $instance['wid_taxExlude']) : 0;
        /* #? Events to show */
        $wid_num_of_p_t_dspl = (isset($instance['wid_num_of_p_t_dspl'])) ? ( $instance['wid_num_of_p_t_dspl']) : 10;
        /* #? Show excerpt or title */
        $wid_erp_exc_or_tit = (isset($instance['wid_erp_exc_or_tit'])) ? ( $instance['wid_erp_exc_or_tit']) : 'post_title';
        /* #? Show post thumbnail */
        $wid_erp_thumb = (isset($instance['wid_erp_thumb'])) ? ( $instance['wid_erp_thumb']) : 1;
        /* #? Hide if no events */
        $wid_erp_gen = (isset($instance['wid_erp_gen'])) ? ($instance['wid_erp_gen']) : 0;
        /* #? Post title text size */
        $wid_erp_pt_s = (isset($instance['wid_erp_pt_s'])) ? ($instance['wid_erp_pt_s']) : 0;
        /* #? Post title text color */
        $wid_erp_pt_c = (isset($instance['wid_erp_pt_c'])) ? ($instance['wid_erp_pt_c']) : '#ffffff';
        /* #? Post title text color use */
        $wid_erp_pt_c_u = (isset($instance['wid_erp_pt_c_u'])) ? ($instance['wid_erp_pt_c_u']) : 0;
        /* #? Post excerpt text size */
        $wid_erp_exc_s = (isset($instance['wid_erp_exc_s'])) ? ($instance['wid_erp_exc_s']) : 0;
        /* #? Post excerpt text color */
        $wid_erp_exc_c = (isset($instance['wid_erp_exc_c'])) ? ($instance['wid_erp_exc_c']) : '#ffffff';
        /* #? Post excerpt text color use */
        $wid_erp_exc_c_u = (isset($instance['wid_erp_exc_c_u'])) ? ($instance['wid_erp_exc_c_u']) : 0;
        /* #? Crop thumbnail */
        $wid_erp_thumb_crop = (isset($instance['wid_erp_thumb_crop'])) ? ($instance['wid_erp_thumb_crop']) : 1;
        /* #? Croped thumbnail height */
        $wid_erp_thumb_h = (isset($instance['wid_erp_thumb_h'])) ? ( $instance['wid_erp_thumb_h']) : 100;
        /* #? Croped thumbnail width */
        $wid_erp_thumb_w = (isset($instance['wid_erp_thumb_w'])) ? ( $instance['wid_erp_thumb_w']) : 200;
        ?>
        <?php /* #? Create nonce */ ?>
        <input
            type="hidden" name="my_meta_box_nonce"
            value="<?php echo wp_create_nonce('my_meta_box_nonce'); ?>" />
        <p>
            <label for="<?php echo $this->get_field_id('wid_erp_title'); ?>"><?php _e('Title:'); ?>
            </label> <input class="erp_wid_opt1" size="39pt"
                            id="<?php echo $this->get_field_id('wid_erp_title'); ?>"
                            name="<?php echo $this->get_field_name('wid_erp_title'); ?>" type="text"
                            value="<?php echo esc_attr($wid_erp_title); ?>" />
        </p>
        <p>
            <label for="<?php echo $this->get_field_id('wid_num_of_p_t_dspl'); ?>"><?php _e('Number of posts to show:'); ?>
            </label> <input class="erp_wid_opt2" size="3pt"
                            id="<?php echo $this->get_field_id('wid_num_of_p_t_dspl'); ?>"
                            name="<?php echo $this->get_field_name('wid_num_of_p_t_dspl'); ?>"
                            type="number" value="<?php echo esc_attr($wid_num_of_p_t_dspl); ?>" /> </p>
        <p>
            <label for="<?php echo $this->get_field_id('wid_getPostsBy'); ?>"><?php echo 'Get posts by: '; ?>
                <select class="erp_wid_opt3"
                        id="<?php echo $this->get_field_id('wid_getPostsBy'); ?>"
                        name="<?php echo $this->get_field_name('wid_getPostsBy'); ?>">
                    <option value="categories"
        <?php selected($wid_getPostsBy, 'categories'); ?>>categories</option>
                    <option value="tags"
                            <?php selected($wid_getPostsBy, 'tags'); ?>>tags</option>
                </select> </label> | 
            <label for="<?php echo $this->get_field_id('wid_taxExlude'); ?>"><?php _e( 'Use taxonomies filter <span title="If this option is enabled widget won\'t span when viewing any post in any categories or tags you have chosen in Settings->Easy Related Posts." id="excluder">*</span>' ); ?></label> 
            <input class="erp_wid_opt5"
                                id="<?php echo $this->get_field_id('wid_taxExlude'); ?>"
                                name="<?php echo $this->get_field_name('wid_taxExlude'); ?>"
                                type="checkbox" <?php echo checked($wid_taxExlude); ?> />
        </p>
        <p><label for="<?php echo $this->get_field_id('wid_erp_gen'); ?>"><?php _e('Hide if no posts to show:'); ?>
            </label> <input class="erp_wid_opt5"
                            id="<?php echo $this->get_field_id('wid_erp_gen'); ?>"
                            name="<?php echo $this->get_field_name('wid_erp_gen'); ?>"
                            type="checkbox" <?php echo checked($wid_erp_gen); ?> />
        </p> <p>
        </p>
        <hr>
        <p style="text-align: center">
            <strong>Thumbnail</strong>
        </p>
        <p>
            <label for="<?php echo $this->get_field_id('wid_erp_thumb'); ?>"><?php _e('Show post thumbnail:'); ?>
            </label> <input class="erp_wid_opt4"
                            id="<?php echo $this->get_field_id('wid_erp_thumb'); ?>"
                            name="<?php echo $this->get_field_name('wid_erp_thumb'); ?>"
                            type="checkbox" <?php echo checked($wid_erp_thumb); ?> />
        </p>
        <p>
            <label for="<?php echo $this->get_field_id('wid_erp_thumb_crop'); ?>"><?php _e('Crop thumbnail:'); ?>
            </label> <input class="erp_wid_opt4"
                            id="<?php echo $this->get_field_id('wid_erp_thumb_crop'); ?>"
                            name="<?php echo $this->get_field_name('wid_erp_thumb_crop'); ?>"
                            type="checkbox" <?php echo checked($wid_erp_thumb_crop); ?> /> : 
            <label style="margin-left: 1%;"	for="<?php echo $this->get_field_id('wid_erp_thumb_h'); ?>"><?php _e('Height:'); ?>
            </label> <input class="erp_wid_opt2" size="3pt"
                            id="<?php echo $this->get_field_id('wid_erp_thumb_h'); ?>"
                            name="<?php echo $this->get_field_name('wid_erp_thumb_h'); ?>"
                            type="number" value="<?php echo esc_attr($wid_erp_thumb_h); ?>" />
            <label style="margin-left: 1%;"	for="<?php echo $this->get_field_id('wid_erp_thumb_w'); ?>"><?php _e('Width:'); ?>
            </label> <input class="erp_wid_opt2" size="3pt"
                            id="<?php echo $this->get_field_id('wid_erp_thumb_w'); ?>"
                            name="<?php echo $this->get_field_name('wid_erp_thumb_w'); ?>"
                            type="number" value="<?php echo esc_attr($wid_erp_thumb_w); ?>" />
        </p>
        <hr>
        <p>
            <label for="<?php echo $this->get_field_id('wid_erp_exc_or_tit'); ?>"><?php echo 'Display post: '; ?>
                <select class="erp_wid_opt3"
                        id="<?php echo $this->get_field_id('wid_erp_exc_or_tit'); ?>"
                        name="<?php echo $this->get_field_name('wid_erp_exc_or_tit'); ?>">
                    <option value="post_title"
        <?php selected($wid_erp_exc_or_tit, 'post_title'); ?>>Title</option>
                    <option value="post_excerpt"
                            <?php selected($wid_erp_exc_or_tit, 'post_excerpt'); ?>>Title + Excerpt</option>
                </select> </label><br>
        </p>
        <p style="text-align: center">
            <strong>Text properties</strong>
        </p>
        <table style="vertical-align: middle;">
            <tr>
                <td>Post title size:</td>
                <td><label for="<?php echo $this->get_field_id('wid_erp_pt_s'); ?>"></label>
                    <input size="1pt"
                           id="<?php echo $this->get_field_id('wid_erp_pt_s'); ?>"
                           name="<?php echo $this->get_field_name('wid_erp_pt_s'); ?>"
                           type="number" value="<?php echo esc_attr($wid_erp_pt_s); ?>" />px
                    <em style="font-size: 9px;"> ('0' for theme default)</em></td>
            </tr>
            <tr>
                <td>Post title color:</td>
                <td><label for="<?php echo $this->get_field_id('wid_erp_pt_c'); ?>"></label>
                    <input class="wp-color-picker-field" data-default-color="#ffffff"
                           size="3pt" id="<?php echo $this->get_field_id('wid_erp_pt_c'); ?>"
                           name="<?php echo $this->get_field_name('wid_erp_pt_c'); ?>"
                           type="text" value="<?php echo esc_attr($wid_erp_pt_c); ?>" /></td>
                <td><label
                        for="<?php echo $this->get_field_id('wid_erp_pt_c_u'); ?>"><?php _e('Use:'); ?>
                    </label> <input class="erp_wid_opt5"
                                    id="<?php echo $this->get_field_id('wid_erp_pt_c_u'); ?>"
                                    name="<?php echo $this->get_field_name('wid_erp_pt_c_u'); ?>"
                                    type="checkbox" <?php echo checked($wid_erp_pt_c_u); ?> /></td>
            </tr>

            <tr>
                <td>Excerpt text size:</td>
                <td><label for="<?php echo $this->get_field_id('wid_erp_exc_s'); ?>"></label>
                    <input size="1pt"
                           id="<?php echo $this->get_field_id('wid_erp_exc_s'); ?>"
                           name="<?php echo $this->get_field_name('wid_erp_exc_s'); ?>"
                           type="number" value="<?php echo esc_attr($wid_erp_exc_s); ?>" />px
                    <em style="font-size: 9px;"> ('0' for theme default)</em></td>
            </tr>
            <tr>
                <td>Excerpt text color:</td>
                <td><label for="<?php echo $this->get_field_id('wid_erp_exc_c'); ?>"></label>
                    <input class="wp-color-picker-field" data-default-color="#ffffff"
                           size="3pt" id="<?php echo $this->get_field_id('wid_erp_exc_c'); ?>"
                           name="<?php echo $this->get_field_name('wid_erp_exc_c'); ?>"
                           type="text" value="<?php echo esc_attr($wid_erp_exc_c); ?>" /></td>
                <td><label
                        for="<?php echo $this->get_field_id('wid_erp_exc_c_u'); ?>"><?php _e('Use:'); ?>
                    </label> <input class="erp_wid_opt5"
                                    id="<?php echo $this->get_field_id('wid_erp_exc_c_u'); ?>"
                                    name="<?php echo $this->get_field_name('wid_erp_exc_c_u'); ?>"
                                    type="checkbox" <?php echo checked($wid_erp_exc_c_u); ?> /></td>
            </tr>
        </table>
        <script type="text/javascript">
            checkThumbOptions = new ChkObj('#<?php echo $this->get_field_id('wid_erp_thumb'); ?>', '#<?php echo $this->get_field_id('wid_erp_thumb_crop'); ?>');
            checkCrop = new ChkObj('#<?php echo $this->get_field_id('wid_erp_thumb_crop'); ?>', '#<?php echo $this->get_field_id('wid_erp_thumb_h'); ?>,#<?php echo $this->get_field_id('wid_erp_thumb_w'); ?>');

            checkThumbOptions.checkBoxChecker();
            checkCrop.checkBoxChecker();

            jQuery(document).ready(function() {
                jQuery('#<?php echo $this->get_field_id("wid_erp_pt_c"); ?>').wpColorPicker();
                jQuery('#<?php echo $this->get_field_id("wid_erp_exc_c"); ?>').wpColorPicker();

                jQuery('#<?php echo $this->get_field_id('wid_erp_thumb'); ?>').click(function() {
                    checkThumbOptions.checkBoxChecker();
                    checkCrop.checkBoxChecker();
                });
                jQuery('#<?php echo $this->get_field_id('wid_erp_thumb_crop'); ?>').click(function() {
                    checkCrop.checkBoxChecker();
                });
            });
        </script>
        <?php
    }

    /**
     * Sanitize widget form values as they are saved.
     *
     * @see WP_Widget::update()
     *
     * @param array $new_instance Values just sent to be saved.
     * @param array $old_instance Previously saved values from database.
     *
     * @return array Updated safe values to be saved.
     */
    public function update($new_instance, $old_instance) {

        /* #? Verify nonce */
        if (!isset($_POST['my_meta_box_nonce']) && !wp_verify_nonce($_POST['my_meta_box_nonce'], 'my_meta_box_nonce')) {
            return;
        }
        $instance = array();

        $instance['wid_erp_title'] = strip_tags($new_instance['wid_erp_title']);
        $instance['wid_num_of_p_t_dspl'] = (is_numeric($new_instance["wid_num_of_p_t_dspl"]) && $new_instance["wid_num_of_p_t_dspl"] > 0) ? ($new_instance['wid_num_of_p_t_dspl'] ) : ($old_instance['wid_num_of_p_t_dspl'] );
        $instance['wid_getPostsBy'] = strip_tags($new_instance['wid_getPostsBy']);
        $instance['wid_erp_exc_or_tit'] = strip_tags($new_instance['wid_erp_exc_or_tit']);
        $instance['wid_erp_thumb'] = (bool) $new_instance["wid_erp_thumb"] ? 1 : 0;
        $instance['wid_erp_gen'] = (bool) $new_instance["wid_erp_gen"] ? 1 : 0;
        $instance['wid_erp_pt_s'] = (is_numeric($new_instance["wid_erp_pt_s"])) ? ($new_instance['wid_erp_pt_s'] ) : ($old_instance['wid_erp_pt_s'] );
        $instance['wid_erp_pt_c'] = strip_tags($new_instance["wid_erp_pt_c"]);
        $instance['wid_erp_pt_c_u'] = (bool) $new_instance["wid_erp_pt_c_u"] ? 1 : 0;
        $instance['wid_erp_exc_s'] = (is_numeric($new_instance["wid_erp_exc_s"])) ? ($new_instance['wid_erp_exc_s'] ) : ($old_instance['wid_erp_exc_s'] );
        $instance['wid_erp_exc_c'] = strip_tags($new_instance["wid_erp_exc_c"]);
        $instance['wid_erp_exc_c_u'] = (bool) $new_instance["wid_erp_exc_c_u"] ? 1 : 0;
        $instance['wid_erp_thumb_crop'] = (bool) $new_instance["wid_erp_thumb_crop"] ? 1 : 0;
        $instance['wid_erp_thumb_h'] = (is_numeric($new_instance["wid_erp_thumb_h"]) && $new_instance["wid_erp_thumb_h"] > 0) ? ($new_instance['wid_erp_thumb_h'] ) : ($old_instance['wid_erp_thumb_h'] );
        $instance['wid_erp_thumb_w'] = (is_numeric($new_instance["wid_erp_thumb_w"]) && $new_instance["wid_erp_thumb_w"] > 0) ? ($new_instance['wid_erp_thumb_w'] ) : ($old_instance['wid_erp_thumb_w'] );
        $instance['wid_taxExlude'] = (bool) $new_instance["wid_taxExlude"] ? 1 : 0;
        
        return $instance;
    }

}

// class Post_Events_Widget
?>